@@ -1,7 +1,7 @@
 /*
  * dev_areca.cpp
  *
- * Home page of code is: http://smartmontools.sourceforge.net
+ * Home page of code is: http://www.smartmontools.org
  *
  * Copyright (C) 2012 Hank Wu <hank@areca.com.tw>
  *
@@ -21,7 +21,7 @@
 #include "dev_interface.h"
 #include "dev_areca.h"
 
-const char * dev_areca_cpp_cvsid = "$Id: dev_areca.cpp 3664 2012-10-24 21:07:11Z chrfranke $"
+const char * dev_areca_cpp_cvsid = "$Id: dev_areca.cpp 4582 2017-11-03 20:54:56Z chrfranke $"
   DEV_ARECA_H_CVSID;
 
 #include "atacmds.h"
@@ -114,7 +114,10 @@ generic_areca_device::~generic_areca_device() throw()
 //   1 if the command succeeded and disk SMART status is "FAILING"
 int generic_areca_device::arcmsr_command_handler(unsigned long arcmsr_cmd, unsigned char *data, int data_len)
 {
-  unsigned int cmds[] =
+  if (arcmsr_cmd >= ARCMSR_CMD_TOTAL)
+    return -1;
+
+  static const unsigned int cmds[ARCMSR_CMD_TOTAL] =
   {
     ARCMSR_IOCTL_READ_RQBUFFER,
     ARCMSR_IOCTL_WRITE_WQBUFFER,
@@ -145,11 +148,6 @@ int generic_areca_device::arcmsr_command_handler(unsigned long arcmsr_cmd, unsig
   sBuf.srbioctl.Timeout = 10000;
   sBuf.srbioctl.ControlCode = cmds[arcmsr_cmd];
 
-  if(arcmsr_cmd >= ARCMSR_CMD_TOTAL)
-  {
-      return -1;
-  }
-
   switch ( arcmsr_cmd )
   {
   // command for writing data to driver
@@ -159,6 +157,7 @@ int generic_areca_device::arcmsr_command_handler(unsigned long arcmsr_cmd, unsig
       sBuf.srbioctl.Length = data_len;
       memcpy((unsigned char *)sBuf.ioctldatabuffer, (unsigned char *)data, data_len);
     }
+    /* FALLTHRU */
     // commands for clearing related buffer of driver
   case ARCMSR_CLEAR_RQBUFFER:
   case ARCMSR_CLEAR_WQBUFFER:
@@ -299,14 +298,14 @@ int generic_areca_device::arcmsr_ui_handler(unsigned char *areca_packet, int are
   if (expected==-3) {
     return set_err(EIO);
   }
-  expected = arcmsr_command_handler(ARCMSR_CLEAR_WQBUFFER, NULL, 0);
+  arcmsr_command_handler(ARCMSR_CLEAR_WQBUFFER, NULL, 0);
   expected = arcmsr_command_handler(ARCMSR_WRITE_WQBUFFER, areca_packet, areca_packet_len);
   if ( expected > 0 )
   {
     expected = arcmsr_command_handler(ARCMSR_READ_RQBUFFER, return_buff, sizeof(return_buff));
   }
 
-  if ( expected < 0 )
+  if ( expected < 3 + 1 ) // Prefix + Checksum
   {
     return -1;
   }
@@ -356,7 +355,8 @@ int generic_areca_device::arcmsr_get_dev_type()
   int ctlr_type = -1;
   int encnum = get_encnum();
   int disknum = get_disknum();
-  unsigned char areca_packet[] = {0x5E, 0x01, 0x61, 0x03, 0x00, 0x22, disknum - 1, encnum - 1, 0x00};
+  unsigned char areca_packet[] = {0x5E, 0x01, 0x61, 0x03, 0x00, 0x22,
+    (unsigned char)(disknum - 1), (unsigned char)(encnum - 1), 0x00};
 
   memset(return_buff, 0, sizeof(return_buff));
   expected = arcmsr_ui_handler(areca_packet, sizeof(areca_packet), return_buff);
@@ -612,10 +612,11 @@ bool generic_areca_device::arcmsr_scsi_pass_through(struct scsi_cmnd_io * iop)
 
   // ----- BEGIN TO SEND TO ARECA DRIVER ------
   expected = arcmsr_ui_handler(areca_packet, areca_packet_len, return_buff);
-  if ( expected < 0 )
-  {
-    return set_err(EIO);
-  }
+
+  if (expected < 0)
+    return set_err(EIO, "arcmsr_scsi_pass_through: I/O error");
+  if (expected < 15) // 7 bytes if port is empty
+    return set_err(EIO, "arcmsr_scsi_pass_through: missing data (%d bytes, expected %d)", expected, 15);
 
   int scsi_status = return_buff[5];
   int in_data_len = return_buff[11] | return_buff[12] << 8 | return_buff[13] << 16 | return_buff[14] << 24;
