@@ -1,9 +1,9 @@
 /* 
  *  os_linux.h
  * 
- * Home page of code is: http://smartmontools.sourceforge.net
+ * Home page of code is: http://www.smartmontools.org
  *
- * Copyright (C) 2003-6 Bruce Allen <smartmontools-support@lists.sourceforge.net>
+ * Copyright (C) 2003-8 Bruce Allen
  *
  * Derived from code that was
  *
@@ -24,8 +24,8 @@
  * any later version.
  *
  * You should have received a copy of the GNU General Public License
- * (for example COPYING); if not, write to the Free
- * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * (for example COPYING); if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  * This code was originally developed as a Senior Thesis by Michael Cornwell
  * at the Concurrent Systems Laboratory (now part of the Storage Systems
@@ -38,7 +38,7 @@
 #ifndef OS_LINUX_H_
 #define OS_LINUX_H_
 
-#define OS_LINUX_H_CVSID "$Id: os_linux.h,v 1.25 2006/08/25 06:06:25 sxzzsf Exp $\n"
+#define OS_LINUX_H_CVSID "$Id: os_linux.h 4431 2017-08-08 19:38:15Z chrfranke $\n"
 
 /* 
    The following definitions/macros/prototypes are used for three
@@ -76,10 +76,7 @@ typedef struct TAG_TW_Passthru {
   } byte0;
   unsigned char size;
   unsigned char request_id;
-  struct { 
-    unsigned char aport:4;
-    unsigned char host_id:4;
-  } byte3;
+  unsigned char unit;
   unsigned char status;  // On return, contains 3ware STATUS register
   unsigned char flags;
   unsigned short param;
